@@ -1,7 +1,7 @@
 //! CKB Tx-pool stores transactions,
 //! design for CKB [Two-Step-Transaction-Confirmation](https://github.com/nervosnetwork/rfcs/blob/master/rfcs/0020-ckb-consensus-protocol/0020-ckb-consensus-protocol.md#Two-Step-Transaction-Confirmation) mechanism
 
-mod block_assembler;
+pub mod block_assembler;
 mod callback;
 mod chunk_process;
 mod component;
@@ -12,6 +12,7 @@ mod process;
 pub mod service;
 mod util;
 
+pub use ckb_jsonrpc_types::BlockTemplate;
 pub use component::entry::TxEntry;
 pub use pool::TxPool;
 pub use process::PlugTarget;
